class Heap:
    def __init__(self):
        self._maxSize = 10 
        self._data = [-1] * self._maxSize
        self._cSize = 0   

    def __len__(self):
        return len(self._data)

    def isEmpty(self):
        return len(self._data) == 0 

    def insert(self,e):
        if self._cSize == self._maxSize-1:
            print("No space in the Heap")
            return    
        self._cSize += 1
        hi = self._cSize
        while hi>1 and e> self._data[hi//2]:
            self._data[hi] = self._data[hi//2]
            hi = hi//2    
        self._data[hi] = e     

    def max(self):
        if self._cSize == 0:
            print("Heap is Empty!")
            return     
        return self._data[1] 

    def delete(self):
        if self._cSize == 0:
            print("Heap is Empty!")
            return 
        e = self._data[1]
        self._data[1] = self._data[self._cSize]
        self._data[self._cSize] = -1
        self._cSize -= 1 
        i = 1
        j = i*2
        while j<self._cSize:
            if self._data[j] < self._data[j+1]:
                j += 1   
            if self._data[i] < self._data[j]:
                self._data[i],self._data[j] = self._data[j] ,self._data[i]
                i = j 
                j = i*2 
            else:
                break 
        return e

def sort(A):
    H = Heap()
    n = len(A)
    for i in range(n):
        H.insert(A[i])
    k = n-1 
    for i in range(H._cSize):
        A[k] = H.delete()
        k -= 1 

if __name__ == "__main__":
    import time
    seconds_1 = time.time()
    a = [9,6,5,8,7,2,3,0,1,4]
    print(sort(a))
    seconds = time.time()
    print("runtime:",seconds-seconds_1)


import numpy as np
class _Node:
    __slots__ = '_element','_next'
    def __init__(self,element,next):
        self._element = element
        self._next = next  
class Queue:
    def __init__(self,N = 'inf'):
        self._front = None 
        self._rare = None 
        self._N = N
        self._size = 0
    
    # Size of linkedlist
    def __len__(self):
        return self._size
    # Print Queue using print function
    def __str__(self):
        if self.isEmpty():
            return "Queue is Empty!"
        s = ""
        p = self._front
        while p:
            s += str(p._element) + " "
            p = p._next
        return s
    # Check if list is empty
    def isEmpty(self):
        return self._size == 0
    # Check if Queue is full
    def isFull(self):
        return self._size == self._N

    # Insert an element to the Queue
    def enqueue(self,e):
        if self.isFull():
            print("Queue is Full!")
            return
        newest = _Node(e,None)
        if self.isEmpty():
            self._front = newest
        else:
            self._rare._next = newest
        self._rare = newest
        self._size += 1

    # Remove an element out of the Queue
    def dequeue(self):
        if self.isEmpty():
            print("Queue is Empty!")
            return
        elif self._size ==1:
            e = self._front._element
            self.clear()
            return e
        else:
            e = self._front._element
            self._front = self._front._next
            self._size -= 1
            return e
        
    # Return first element of the Queue
    def first(self):
        if self.isEmpty():
            print("Queue is Empty!")
            return
        else:
            return self._front._element

    # Display the Queue
    def display(self):
        if self.isEmpty():
            return "Queue is Empty!"
        s = ""
        p = self._front
        while p:
            s += str(p._element)+" | "
            p = p._next
        print(s)
    
    # Delete the Queue
    def clear(self):
        self._size = 0
        self._front = None
class Graph:
    def __init__(self,vertices):
        self._vertices = vertices
        self._adjMat = np.zeros((vertices,vertices))
        self._visited = [0] * vertices

    def insertEdge(self,u,v,x=1):
        self._adjMat[u][v] = x   

    def removeEdge(self,u,v):
        self._adjMat[u][v] = 0

    def existEdge(self,u,v):
        return self._adjMat[u][v] != 0 

    def vertexCount(self):
        return self._vertices

    def edgeCount(self):
        count = 0
        for i in range(self._vertices):
            for j in range(self._vertices):
                if self._adjMat[i][j] != 0:
                    count += 1
        return count   

    def vertices(self):
        for i in range(self._vertices):
            print(i,end=" ")
        print()  

    def edges(self):
        for i in range(self._vertices):
            for j in range(self._vertices):
                if self._adjMat[i][j] != 0:
                    print(i,"--",j)

    def outDegree(self,v):
        count = 0
        for j in range(self._vertices):
            if self._adjMat[v][j] != 0:
                 count += 1
        return count 

    def inDegree(self,v):
        count = 0 
        for i in range(self._vertices):
            if self._adjMat[i][v] != 0:
                count += 1
        return count   

    def display(self):
        print(self._adjMat)

    def BFS(self,s):
        i = s
        q = Queue()
        visited = [0] * self._vertices
        print(i,end=' ')
        visited[i] = 1
        q.enqueue(i)
        while not q.isEmpty():
            i = q.dequeue()
            for j in range(self._vertices):
                if self._adjMat[i][j] == 1 and visited[j] == 0:
                    print(j,end=' ')
                    visited[j] = 1
                    q.enqueue(j)
    def DFS(self,s):
        if self._visited[s]==0:
            print(s,end=' ')
            self._visited[s]=1
            for j in range(self._vertices):
                if self._adjMat[s][j]==1 and self._visited[j]==0:
                    self.DFS(j)

if __name__ == '__main__':
    # Unweighted Undirected
    G = Graph(4)
    G.display()
    print(G.vertexCount())
    print(G.edgeCount())
    G.insertEdge(0,1)
    G.insertEdge(1,0)
    G.insertEdge(0,2)
    G.insertEdge(2,0)
    G.insertEdge(1,2)
    G.insertEdge(2,1)
    G.insertEdge(3,2)
    G.insertEdge(2,3)
    G.display()
    print(G.vertexCount())
    print("Edges:",G.edgeCount())
    print("Out Degree:",G.outDegree(2))
    print("In Degree",G.inDegree(2))
    G.edges()
    print(G.existEdge(1,3))
    G.removeEdge(1,2)
    G.edges()
    print("Edges:",G.edgeCount())
    print(G.existEdge(1,2))

    # Weighted Undirected
    G = Graph(4)
    G.display()
    print(G.vertexCount())
    print(G.edgeCount())
    G.insertEdge(0,1,26)
    G.insertEdge(1,0,26)
    G.insertEdge(0,2,16)
    G.insertEdge(2,0,16)
    G.insertEdge(1,2,12)
    G.insertEdge(2,1,12)
    G.insertEdge(3,2,8)
    G.insertEdge(2,3,8)
    G.display()
    print(G.vertexCount())
    print("Edges:",G.edgeCount())
    print("Out Degree:",G.outDegree(2))
    print("In Degree",G.inDegree(2))
    G.edges()
    print(G.existEdge(1,3))
    G.removeEdge(1,2)
    G.edges()
    print("Edges:",G.edgeCount())
    print(G.existEdge(1,2))

    # Unweighted Directed
    G = Graph(4)
    G.display()
    print(G.vertexCount())
    print(G.edgeCount())
    G.insertEdge(0,1)
    G.insertEdge(0,2)
    G.insertEdge(1,2)
    G.insertEdge(2,3)
    G.display()
    print(G.vertexCount())
    print("Edges:",G.edgeCount())
    print("Out Degree:",G.outDegree(2))
    print("In Degree",G.inDegree(2))
    G.edges()
    print(G.existEdge(1,3))
    G.removeEdge(1,2)
    G.edges()
    print("Edges:",G.edgeCount())
    print(G.existEdge(1,2))

    # Weighted Directed
    G = Graph(4)
    G.display()
    print(G.vertexCount())
    print(G.edgeCount())
    G.insertEdge(0,1,26)
    G.insertEdge(0,2,16)
    G.insertEdge(1,2,12)
    G.insertEdge(2,3,8)
    G.display()
    print(G.vertexCount())
    print("Edges:",G.edgeCount())
    print("Out Degree:",G.outDegree(2))
    print("In Degree",G.inDegree(2))
    G.edges()
    print(G.existEdge(1,3))
    G.removeEdge(1,2)
    G.edges()
    print("Edges:",G.edgeCount())
    print(G.existEdge(1,2))
    print(G.BFS(0))
    print(G.DFS(0))


class Item:
    __slots__="value","weight"
    def __init__(self,value,weight):
        self.value = value
        self.weight = weight

def fks(capacity,Items,n):
        a = [0]*n
        sack = [0]*n
        for i in range (0,n):
            a[i] = (Items[i].value/Items[i].weight)
        while a:
            index = a.index(max(a))
            if capacity>Items[index].weight:
                sack[index] = (Items[index].value)
                capacity -= Items[index].weight
                a[index] = -1
            else:
                break
        if a and capacity>0:
            index = a.index(max(a))
            sack[index]=((Items[index].value*capacity)/Items[index].weight)
        return sum(sack)

if __name__ == "__main__":
    a = Item(60,10)
    b = Item(100,20)
    c = Item(120,30)
    A = [a,b,c] 
    print(fks(50,A,len(A)))


class Hash():
    def __init__(self):
        self.hashSize = 10    
        self.hashTable = [0]* self.hashSize

    def hashCode(self,key):
        return key%self.hashSize

    def lProbe(self,e):
        i = self.hashCode(e)
        j = 0 
        while self.hashTable[(i+j)%self.hashSize] != 0:
            j += 1       
        return (i+j)%self.hashSize
    
    def insert(self,e):
        i = self.hashCode(e)
        if self.hashTable[i] == 0:
            self.hashTable[i] = e 
        else: 
            i = self.lProbe(e)
            self.hashTable[i] = e 
        
    def search(self,key):
        i = self.hashCode(key)
        j = 0
        while self.hashTable[(i+j)%self.hashSize] != key:
            if self.hashTable[(i+j)%self.hashSize] == 0:
                return False
            j += 1
        return True 
    
    def display(self):
        print(self.hashTable)

if __name__ == '__main__':
    H = Hash()
    H.insert(15)
    H.insert(92)
    H.insert(35)
    H.insert(56)
    H.insert(28)
    H.insert(12)
    H.insert(26)
    H.insert(46)
    H.display()
    print(H.search(35))
    print(H.search(36))


def insertionSort(A):
    n = len(A)
    for i in range(1,n):
        cvalue = A[i]
        position = i
        while position > 0 and A[position-1] > cvalue:
            A[position] = A[position-1]
            position = position - 1
        A[position] = cvalue
def sort(A):
    n = len(A)
    maxElement = max(A)
    l = []
    buckets = [l] * 10
    for i in range(n):
        j = int(n * A[i] / (maxElement + 1) )
        if len(buckets[j]) == 0:
            buckets[j] = [A[i]]
        else:
            buckets[j].append(A[i])
    for i in range(10):
        insertionSort(buckets[i])
    k = 0
    for i in range(10):
        for j in range(len(buckets[i])):
            A[k] = buckets[i].pop(0)
            k = k + 1
    return A
if __name__ == "__main__":
    import time
    seconds_1 = time.time()
    a = [9,6,5,8,7,2,3,0,1,4]
    print(sort(a))
    seconds = time.time()
    print("runtime:",seconds-seconds_1)


def select(Arr):
    a = []
    last_end = float("-inf")
    Arr.sort(key = lambda i : i[1])
    for i in Arr:
        start = i[0]
        if start > last_end:
            last_end = i[1]
            a.append(i)
    return a
if __name__ == '__main__':
    Arr = [[0,6],[4,7],[3,9],[8,11]]
    print(select(Arr))


class Queue:
    def __init__(self,N = 'inf'):
        self._data = [] 
        self._N = N  

    # Size of Queue
    def __len__(self):
        return len(self._data)
    # Print Queue using print function
    def __str__(self):
        if self.isEmpty():
            return "Queue is Empty!"
        s = ""
        for i in self._data:
            s += str(i) + " "
        return s
    # Check if Queue is empty
    def isEmpty(self):
        return len(self._data) == 0
    # Check if Queue is full
    def isFull(self):
        return len(self._data) == self._N

    # Insert an element to the Queue
    def enqueue(self,e):
        if self.isFull():
            print("Queue is Full!")
            return
        else:
            self._data.append(e)

    # Remove an element out of the Queue
    def dequeue(self):
        if self.isEmpty():
            print("Queue is Empty!")
            return
        else:
            return self._data.pop(0)

    # Return first element of the Queue
    def first(self):
        if self.isEmpty():
            print("Queue is Empty!")
            return
        else:
            return self._data[0]
    
    # Display the Queue
    def display(self):
        for i in self._data:
            print(i,"|",end=" ")
        print()
    
    # Delete the Queue
    def clear(self):
        self._data = []
class _Node:
    __slots__='_element','_left','_right'

    def __init__(self,element,left=None,right=None):
        self._element = element
        self._left = left
        self._right = right

class Tree:
    def __init__(self):
        self._root = None

    def recInsert(self,tRoot,e):
        if tRoot:
            if e==tRoot._element:
                return tRoot
            elif e < tRoot._element:
                tRoot._left = self.recInsert(tRoot._left,e)
            elif e > tRoot._element:
                tRoot._right = self.recInsert(tRoot._right,e)
        else:
            n = _Node(e)
            tRoot = n 
        return tRoot

    def insert(self,tRoot,e):
        temp = None
        while tRoot:
            temp = tRoot
            if e==tRoot._element:
                return
            elif e < tRoot._element:
                tRoot = tRoot._left
            elif e > tRoot._element:
                tRoot = tRoot._right
        n = _Node(e)
        if self._root:
            if e < temp._element:
                temp._left = n 
            else:
                temp._right = n 
        else:
            self._root = n  

    
    def search(self,key):
        troot = self._root
        while troot:
            if key == troot._element:
                return True 
            elif key < troot._element:
                troot = troot._left
            elif key > troot._element:
                troot = troot._right 
        return False
    
    def recSearch(self,troot,key):
        if troot:
            if key == troot._element:
                return True
            elif key < troot._element:
                return self.recSearch(troot._left,key) 
            elif key > troot._element:
                return self.recSearch(troot._right,key) 
        else:
            return False
    def inOrder(self,tRoot):
        if tRoot:
            self.inOrder(tRoot._left)
            print(tRoot._element,end=" ")
            self.inOrder(tRoot._right)
    def preOrder(self,tRoot):
        if tRoot:
            print(tRoot._element,end=" ")
            self.preOrder(tRoot._left)
            self.preOrder(tRoot._right)
    def postOrder(self,tRoot):
        if tRoot:
            self.postOrder(tRoot._left)
            self.postOrder(tRoot._right)
            print(tRoot._element,end=" ")
    def levelOrder(self):
        q = Queue()
        t = self._root
        print(t._element,end=" ")
        q.enqueue(t)
        while not q.isEmpty():
            t = q.dequeue()
            if t._left:
                print(t._left._element,end=" ")
                q.enqueue(t._left)
            if t._right:
                print(t._right._element,end=" ")
                q.enqueue(t._right)
        
    def delete(self,e):
        p = self._root
        pp = None 
        while p and p._element != e:
            pp = p 
            if e < p._element:
                p = p._left
            else:
                p = p._right 
        if not p:
            return False
        if p._left and p._right:
            s = p._left 
            ps = p 
            while s._right:
                ps = s 
                s = s._right 
            p._element = s._element
            p = s 
            pp = ps 
        c = None 
        if p._left:
            c = p._left 
        else:
            c = p._right
        if p == self._root:
            self._root = None
        else:
            if p == pp._left:
                pp._left = c 
            else: 
                pp._right = c   

        

    def count(self,tRoot):
        if tRoot:
            x = self.count(tRoot._left)   
            y = self.count(tRoot._right)   
            return x+y+1
        return 0

    def calHeight(self,tRoot):
        if tRoot:
            x = self.calHeight(tRoot._left)   
            y = self.calHeight(tRoot._right)   
            if x > y:
                return x+1
            else:
                return y+1
        return 0

    def height(self,tRoot):
        h = self.calHeight(tRoot)
        return h-1
if __name__ == '__main__':
    x = Tree()
    y = Tree()
    z = Tree()
    r = Tree()
    s = Tree()
    t = Tree()
    a = Tree()
    t._root = t.recInsert(t._root,70)
    t.insert(t._root,40)
    t.insert(t._root,60)
    t.insert(t._root,20)
    t.insert(t._root,50)
    t.recInsert(t._root,30)
    t.recInsert(t._root,10)
    print("In order:")
    t.inOrder(t._root)
    print()
    print("Pre order:")
    t.preOrder(t._root)
    print()
    print("Post order:")
    t.postOrder(t._root)
    print()
    print("Post order:")
    t.levelOrder()
    print()
    print("No of Nodes:",end=" ")
    print(t.count(t._root))
    print("Height of tree:",end=" ")
    print(t.height(t._root))
    print(t.search(60))
    print(t.search(55))
    print(t.recSearch(t._root,40))
    print(t.recSearch(t._root,45))
    t.delete(20)
    t.inOrder(t._root)
    print()
    t.delete(70)
    t.inOrder(t._root)